<?php

namespace LiamW\PostMacros\ControllerPlugin;

use LiamW\PostMacros\Entity\Macro;
use XF\ControllerPlugin\AbstractPlugin;
use XF\Mvc\ParameterBag;

class GlobalMacroPermission extends AbstractPlugin
{
	protected $viewFormatter = 'LiamW\PostMacros:Permission\GlobalMacro%s';
	protected $templateFormatter = 'liamw_postmacros_permission_globalmacro_%s';
	protected $routePrefix = 'permissions/global-macros';

	public function setRoutePrefix($prefix)
	{
		$this->routePrefix = $prefix;
	}

	public function actionList(ParameterBag $params)
	{
		$macro = $this->assertGlobalMacroExists($params->macro_id);

		/** @var \XF\Repository\PermissionEntry $entryRepo */
		$entryRepo = $this->repository('XF:PermissionEntry');
		$entries = $entryRepo->getContentPermissionEntriesGrouped('liamw_macros_globalmacro', $macro->macro_id);

		if ($entries['users'])
		{
			$users = $this->finder('XF:User')
				->whereIds(array_keys($entries['users']))
				->order('username')
				->fetch();
		}
		else
		{
			$users = [];
		}

		/** @var \XF\Repository\UserGroup $userGroupRepo */
		$userGroupRepo = $this->repository('XF:UserGroup');
		$userGroups = $userGroupRepo->findUserGroupsForList()->fetch();

		$viewParams = [
			'macro' => $macro,
			'entries' => $entries,
			'userGroups' => $userGroups,
			'users' => $users
		];

		return $this->view(
			$this->formatView('List'),
			$this->formatTemplate('list'),
			$viewParams
		);
	}

	public function actionEdit(ParameterBag $params, $type = null)
	{
		$macro = $this->assertGlobalMacroExists($params->macro_id);

		if ($type === null)
		{
			$type = $this->filter('type', 'str');
			if (!$type)
			{
				if ($this->filter('user_group_id', 'uint'))
				{
					$type = 'user_group';
				}
				else if ($this->filter('user_id', 'uint'))
				{
					$type = 'user';
				}
			}
		}

		/** @var \XF\Repository\Permission $permissionRepo */
		$permissionRepo = $this->repository('XF:Permission');
		$permissionData = $permissionRepo->getContentPermissionListData('liamw_macros_globalmacro');

		/** @var \XF\Repository\PermissionEntry $entryRepo */
		$entryRepo = $this->repository('XF:PermissionEntry');
		$entries = $entryRepo->getContentPermissionEntriesGrouped('liamw_macros_globalmacro', $macro->macro_id);

		if ($type == 'user_group')
		{
			$userGroup = $this->assertRecordExists('XF:UserGroup', $this->filter('user_group_id', 'uint'));
			$user = null;

			$typeEntries = isset($entries['groups'][$userGroup->user_group_id])
				? $entries['groups'][$userGroup->user_group_id]
				: [];

			$saveParams = [
				'type' => 'user_group',
				'user_group_id' => $userGroup->user_group_id
			];
		}
		else if ($type == 'user')
		{
			$username = $this->filter('username', 'str');
			if ($username)
			{
				$user = $this->em()->findOne('XF:User', ['username' => $username]);
				if (!$user)
				{
					return $this->error(\XF::phrase('requested_user_not_found'), 404);
				}

				return $this->redirect($this->buildLink(
					$this->routePrefix . '/edit',
					$macro,
					['user_id' => $user->user_id]
				));
			}

			$user = $this->assertRecordExists('XF:User', $this->filter('user_id', 'uint'));
			$userGroup = null;

			$typeEntries = isset($entries['users'][$user->user_id])
				? $entries['users'][$user->user_id]
				: [];

			$saveParams = [
				'type' => 'user',
				'user_id' => $user->user_id
			];
		}
		else
		{
			return $this->notFound();
		}

		$viewParams = [
			'macro' => $macro,
			'userGroup' => $userGroup,
			'user' => $user,
			'permissionData' => $permissionData,
			'typeEntries' => $typeEntries,
			'saveParams' => $saveParams
		];

		return $this->view(
			$this->formatView('Edit'),
			$this->formatTemplate('edit'),
			$viewParams
		);
	}

	public function actionSave(ParameterBag $params, $type = null)
	{
		$this->assertPostOnly();

		$macro = $this->assertGlobalMacroExists($params->macro_id);

		if ($type === null)
		{
			$type = $this->filter('type', 'str');
		}

		if ($type == 'user_group')
		{
			$this->saveUserGroup($macro);
		}
		else if ($type == 'user')
		{
			$this->saveUser($macro);
		}

		return $this->redirect($this->buildLink($this->routePrefix, $macro));
	}

	protected function saveUserGroup(Macro $macro)
	{
		/** @var \XF\Entity\UserGroup $userGroup */
		$userGroup = $this->assertRecordExists('XF:UserGroup', $this->filter('user_group_id', 'uint'));
		$permissions = $this->filter('permissions', 'array');

		/** @var \XF\Service\UpdatePermissions $permissionUpdater */
		$permissionUpdater = $this->service('XF:UpdatePermissions');
		$permissionUpdater->setContent('liamw_macros_globalmacro', $macro->macro_id)->setUserGroup($userGroup);
		$permissionUpdater->updatePermissions($permissions);
	}

	protected function saveUser(Macro $macro)
	{
		/** @var \XF\Entity\User $user */
		$user = $this->assertRecordExists('XF:User', $this->filter('user_id', 'uint'));
		$permissions = $this->filter('permissions', 'array');

		/** @var \XF\Service\UpdatePermissions $permissionUpdater */
		$permissionUpdater = $this->service('XF:UpdatePermissions');
		$permissionUpdater->setContent('liamw_macros_globalmacro', $macro->macro_id)->setUser($user);
		$permissionUpdater->updatePermissions($permissions);
	}

	protected function formatView($type)
	{
		$formatter = $this->viewFormatter;

		if (is_string($formatter))
		{
			return sprintf($formatter, $type);
		}
		else if ($formatter instanceof \Closure)
		{
			return $formatter($type);
		}
		else
		{
			return '';
		}
	}

	protected function formatTemplate($type)
	{
		$formatter = $this->templateFormatter;

		if (is_string($formatter))
		{
			return sprintf($formatter, $type);
		}
		else if ($formatter instanceof \Closure)
		{
			return $formatter($type);
		}
		else
		{
			return '';
		}
	}

	/**
	 * @param string            $id
	 * @param array|string|null $with
	 * @param null|string       $phraseKey
	 *
	 * @return \LiamW\PostMacros\Entity\Macro
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertGlobalMacroExists($id, $with = null, $phraseKey = null)
	{
		/** @var \LiamW\PostMacros\Entity\Macro $macro */
		$macro = $this->controller->assertRecordExists('LiamW\PostMacros:Macro', $id, $with, $phraseKey);
		if ($macro->type != 'global')
		{
			if (!$phraseKey)
			{
				$phraseKey = 'requested_page_not_found';
			}

			throw $this->exception(
				$this->notFound(\XF::phrase($phraseKey))
			);
		}

		return $macro;
	}
}