<?php

namespace LiamW\PostMacros\Entity;

use function GuzzleHttp\Psr7\str;
use LiamW\PostMacros\MetaData\Handler\AbstractHandler;
use XF\BbCode\RenderableContentInterface;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;
use XF\Mvc\View;

/**
 * COLUMNS
 *
 * @property int                                             macro_id
 * @property int|null                                        user_id
 * @property string                                          type
 * @property string                                          title
 * @property string                                          content
 * @property bool                                            auto_submit
 * @property array                                           change_category
 * @property array                                           metadata
 *
 * GETTERS
 * @property mixed                                           custom_tokens
 * @property mixed                                           rendered_content
 * @property mixed                                           rendered_metadata
 *
 * RELATIONS
 * @property \XF\Entity\User                                 Owner
 * @property \LiamW\PostMacros\Entity\GlobalMacroEnrolment[] Enrolment
 * @property \LiamW\PostMacros\Entity\MacroSharing[]         Sharing
 * @property \LiamW\PostMacros\Entity\MacroSharing           VisitorSharing
 * @property \LiamW\PostMacros\Entity\GlobalMacroEnrolment   VisitorEnrolment
 * @property \XF\Entity\PermissionCacheContent[]             Permissions
 */
class Macro extends Entity implements RenderableContentInterface
{
	protected $contextUrl;
	protected $customTokens;
    protected $categoryId;

	public function canEdit(&$error = null)
	{
		if ($this->type == 'global')
		{
			return false;
		}

		$visitor = \XF::visitor();

		if ($visitor->user_id == $this->user_id)
		{
			return true;
		}

		if ($this->Sharing)
		{
			/** @var \LiamW\PostMacros\Entity\MacroSharing $sharingRecord */
			foreach ($this->Sharing as $sharingRecord)
			{
				if ($sharingRecord->user_id == $visitor->user_id && $sharingRecord->can_write)
				{
					return true;
				}
			}
		}

		return false;
	}

	public function canDelete(&$error = null)
	{
		if ($this->type == 'global')
		{
			return false;
		}

		$visitor = \XF::visitor();

		return $visitor->user_id == $this->user_id;
	}

	public function canEditSharings(&$error = null)
	{
		if ($this->type == 'global')
		{
			return false;
		}

		$visitor = \XF::visitor();

		return $visitor->hasPermission('liamw_postmacros', 'share') && $visitor->user_id == $this->user_id;
	}

	public function getCustomTokens()
	{
		preg_match_all("/\{\?\+(?P<required>!?)(?P<label>.+)\}/U", $this->content, $matches, PREG_SET_ORDER);

		$tokens = [];
        $exists = [];
		foreach ($matches AS $match)
		{
            if(!in_array(trim($match['label']), $exists))
            {
                $exists[] = trim($match['label']);
                $tokens[] = ['label' => $match['label'], 'required' => $match['required'] === '!'];
            }
		}

		return $tokens;
	}

	public function getRenderedContent()
	{
	    $category  = '';
	    if($this->categoryId)
        {
            $category = "\n" .'{+?id ' . $this->categoryId .'}' . "\n";
        }
        $content = $this->getMacroRepository()->replaceCustomPlaceholders($this, $this->customTokens);
        $content = $this->getMacroRepository()->renderMacroPlaceholders($content, $this->contextUrl);
	    return $content . $category;

	}

	public function getRenderedMetadata()
	{
		$renderedMetadata = [];

		$view = new View($this->app()->renderer('public'), $this->app()->response());

		foreach ($this->metadata AS $class => $value)
		{
			if (empty($value))
			{
				continue;
			}

			try
			{
				/** @var AbstractHandler $obj */
				$obj = AbstractHandler::create($class, null, $value);

				if ($obj)
				{
					$renderedMetadata[] = $obj->renderView($view);
				}
			}
			catch (\Exception $e)
			{
				\XF::logException($e);
			}
		}

		return $renderedMetadata;
	}

	public function getBbCodeRenderOptions($context, $type)
	{
		return ['entity' => $this, 'user' => $this->user_id ? $this->Owner : null];
	}

	public function setContextUrl($actionUrl)
	{
		$this->contextUrl = $actionUrl;

		return $this;
	}

	public function setCustomTokens($customTokens)
	{
		$this->customTokens = $customTokens;

		return $this;
	}
    public function setChangeCategoryId($id)
    {
        $this->categoryId = $id;
        return $this;
    }
	public function getNewSharing()
	{
		$sharing = $this->_em->create('LiamW\PostMacros:MacroSharing');

		$sharing->macro_id = $this->_getDeferredValue(function ()
		{
			return $this->macro_id;
		}, 'save');

		return $sharing;
	}

	public function getNewEnrolment()
	{
		if ($this->type != 'global')
		{
			throw new \LogicException("Enrolments can only be associated with global type macros.");
		}

		$enrolment = $this->_em->create('LiamW\PostMacros:GlobalMacroEnrolment');

		$enrolment->macro_id = $this->_getDeferredValue(function ()
		{
			return $this->macro_id;
		}, 'save');

		return $enrolment;
	}

	public function canUnenroll()
	{
		$userId = \XF::visitor()->user_id;

		return ($this->type == 'global' && $this->Enrolment[$userId]) || ($this->type == 'user' && $this->Sharing[$userId] && $this->Sharing[$userId]->enrolled);
	}

	public function canEnroll(&$error = '', $permissionOnly = false)
	{
		$canEnroll = false;
		$userId = \XF::visitor()->user_id;

		switch ($this->type)
		{
			case 'user':
				if (!$permissionOnly)
				{
					if ($this->Sharing[$userId] && $this->Sharing[$userId]->enrolled)
					{
						$error = \XF::phraseDeferred('liamw_postmacros_you_are_already_enrolled_onto_this_shared_macro');

						break;
					}

					if (!$this->Sharing[$userId])
					{
						break;
					}
				}

				$canEnroll = \XF::visitor()->hasPermission('liamw_postmacros', 'enroll');
				break;
			case 'global':
				if (!$permissionOnly && $this->Enrolment[$userId])
				{
					$error = \XF::phraseDeferred('liamw_postmacros_you_are_already_enrolled_onto_this_global_macro');

					break;
				}

				$canEnroll = \XF::visitor()->hasContentPermission('liamw_macros_globalmacro', $this->macro_id, 'enroll');
				break;
		}

		return $canEnroll;
	}

    /**
     * @param $url
     * @return array|int
     * @throws \Exception
     */
    public function getCategory($url)
    {

        $entity = $this->getEntityByType($url);
        if($entity === 'posts')
        {
            $entity = 'threads';
        }
        if(isset($this->change_category[$entity]) && !empty($entity))
        {
            $changeCategory = $this->change_category[$entity];
            $handlerChangeCategory = $this->getMacroRepository()->getChangeTypeHandler($changeCategory, '');
            if($handlerChangeCategory->canChangeCategory())
            {
                return [
                    'categories' => $handlerChangeCategory->getCategory(),
                    'label' => $handlerChangeCategory->label()
                ];
            }
        }
        return empty($entity);
    }
    protected function getEntityByType($url)
    {
        return preg_replace('#.*(\/|\?)(threads|posts)\/(.*)#', '$2', $url);
    }
	public function enroll()
	{
		$userId = \XF::visitor()->user_id;

		switch ($this->type)
		{
			case 'user':
				$this->Sharing[$userId]->enrolled = 1;
				$this->Sharing[$userId]->save();
				break;
			case 'global':
				$enrolment = $this->getNewEnrolment();
				$enrolment->user_id = \XF::visitor()->user_id;
				$enrolment->save();

				break;
		}

		return $this;
	}

	public function unenroll()
	{
		$userId = \XF::visitor()->user_id;

		switch ($this->type)
		{
			case 'user':
				$this->Sharing[$userId]->enrolled = 0;
				$this->Sharing[$userId]->save();
				break;
			case 'global':
				$this->Enrolment[\XF::visitor()->user_id]->delete();
				break;
		}

		return $this;
	}

    public function getUserRender()
    {
        preg_match("/\{=\+(?P<username>.+)\}/", $this->content, $matches);
        $user = $this->finder('XF:User')->where('username', '=', $matches['username'])->fetchOne();
        return $user;
    }
    public function getRedirection()
    {
        if($this->categoryId)
        {
            return true;
        }
        return false;
    }
    public function getListChangeCategory()
    {
        return [
            'threads' => [
                'handler' => 'LiamW\PostMacros:Thread',
                'label' => \XF::phrase('thread')
            ]
        ];
    }
	public static function getStructure(Structure $structure)
	{
		$structure->table = 'xf_liamw_post_macros';
		$structure->primaryKey = 'macro_id';
		$structure->columns = [
			'macro_id' => ['type' => self::UINT, 'autoIncrement' => true],
			'user_id' => ['type' => self::UINT, 'required' => true, 'nullable' => true],
			'type' => [
				'type' => self::STR,
				'allowedValues' => ['user', 'global'],
				'default' => 'user'
			],
			'title' => [
				'type' => self::STR,
				'required' => 'liamw_postmacros_please_enter_a_valid_title',
				'maxLength' => 150
			],
			'content' => [
				'type' => self::STR,
				'required' => 'liamw_postmacros_please_enter_a_valid_message'
			],
			'auto_submit' => ['type' => self::BOOL, 'default' => 0],
			'display_order' => ['type' => self::UINT, 'default' => 0],
            'change_category' => ['type' => self::JSON_ARRAY, 'default' => []],
			'metadata' => ['type' => self::JSON_ARRAY, 'default' => []],
		];
		$structure->getters = [
		    'custom_tokens' => true,
            'rendered_content' => true,
            'rendered_metadata' => true,
            'user_render' => true,
            'redirection' => true,
            'list_change_category' => true,
        ];
		$structure->relations = [
			'Owner' => ['entity' => 'XF:User', 'type' => self::TO_ONE, 'conditions' => 'user_id'],
			'Enrolment' => [
				'entity' => 'LiamW\PostMacros:GlobalMacroEnrolment',
				'type' => self::TO_MANY,
				'conditions' => 'macro_id',
				'key' => 'user_id',
				'cascadeDelete' => true
			],
			'Sharing' => [
				'entity' => 'LiamW\PostMacros:MacroSharing',
				'type' => self::TO_MANY,
				'conditions' => 'macro_id',
				'key' => 'user_id',
				'cascadeDelete' => true
			],
			'Permissions' => [
				'entity' => 'XF:PermissionCacheContent',
				'type' => self::TO_MANY,
				'conditions' => [
					['$type', '=', 'global'],
					['content_type', '=', 'liamw_macros_globalmacro'],
					['content_id', '=', '$macro_id']
				],
				'key' => 'permission_combination_id',
				'proxy' => true
			]
		];
		$structure->withAliases = [
			'list' => [
				function ()
				{
					$permissionCombinationId = \XF::visitor()->permission_combination_id;
					$userId = \XF::visitor()->user_id;

					return [
						'Owner',
						"Sharing|$userId",
						"Enrolment|$userId",
						"Permissions|$permissionCombinationId"
					];
				}
			]
		];

		return $structure;
	}

	protected final function verifyMetadata(&$metadata, $key, $type, $columnOptions)
	{
		// No metadata somehow.
		if (!is_array($metadata))
		{
			$metadata = [];

			return true;
		}

		$invalidMetadata = $validMetadata = [];

		foreach ($this->getMacroRepository()->getMetaDataHandlersGrouped($this->metadata) as $metaDataHandlers)
		{
			foreach ($metaDataHandlers AS $metaDataHandler)
			{
				/** @var AbstractHandler $metaDataHandler */

				if (!isset($metadata[$metaDataHandler->getShortName()]))
				{
					continue;
				}

				$newValue = $metadata[$metaDataHandler->getShortName()];

				$metadataValid = $metaDataHandler->verifyValue($this, $newValue);

				if ($metadataValid)
				{
					$validMetadata[$metaDataHandler->getShortName()] = $newValue;
				}
				else
				{
					$invalidMetadata[] = $metaDataHandler->getShortName();
				}
			}
		}

		if ($invalidMetadata)
		{
			$this->error(\XF::phrase('liamw_postmacros_one_or_more_metadata_values_invalid'), $key);

			return false;
		}

		$metadata = $validMetadata;

		return true;
	}

	protected function _postSave()
	{
		if ($this->isInsert() && $this->type == 'global')
		{
			$this->app()->jobManager()->enqueueUnique('permissionRebuild', 'XF:PermissionRebuild');
		}
	}

	/**
	 * @return \LiamW\PostMacros\Repository\Macro|\XF\Mvc\Entity\Repository
	 */
	protected function getMacroRepository()
	{
		return $this->repository('LiamW\PostMacros:Macro');
	}
}