<?php

namespace LiamW\PostMacros\MetaData\Handler;

use LiamW\PostMacros\Entity\Macro;
use XF\Mvc\Entity\Entity;
use XF\Mvc\View;

abstract class AbstractHandler
{
	protected $shortName = null;

	protected $group;

	protected $value;

	/**
	 * MetaData constructor. Classes should be instantiated using the {@link create} method.
	 *
	 * @param mixed $value Metadata value
	 * @param \LiamW\PostMacros\MetaData\Group\AbstractGroup $group
	 */
	protected function __construct($value, $group)
	{
		if (!$this->shortName)
		{
			throw new \BadMethodCallException("Metadata implementations must override the shortName property");
		}

		$this->group = $group;
		$this->value = $value;
	}

	/**
	 * @param mixed $value
	 */
	public function setValue($value)
	{
		$this->value = $value;
	}

	/**
	 * @return \LiamW\PostMacros\MetaData\Group\AbstractGroup
	 */
	public function getGroup()
	{
		return $this->group;
	}

	public final function renderInput(View $view)
	{
		return $this->renderMetaDataInput($view, 'metadata[' . $this->shortName . ']');
	}

	public final function renderView(View $view)
	{
		return $this->renderMetaData($view);
	}

	public final function verifyValue(Macro $macro, &$newValue)
	{
		return $this->verifyMetadataValue($macro, $newValue);
	}

	public final function getShortName()
	{
		return $this->shortName;
	}

	public final function process(Entity $entity, $value)
	{
		$this->processInternal($entity, $value);
	}

	/**
	 * Return the HTML for the input related to this metadata when editing a macro. The input name should be the
	 * $inputName variable.
	 *
	 * @param \XF\Mvc\View $view
	 * @param string $inputName The name of the form input.
	 *
	 * @return mixed
	 */
	abstract protected function renderMetaDataInput(View $view, $inputName);

	/**
	 * Return the HTML for the input related to this metadata when viewing a macro in the list.
	 *
	 * @param \XF\Mvc\View $view
	 *
	 * @return mixed
	 * @internal param string $inputName The name of the form input.
	 */
	abstract protected function renderMetaData(View $view);

	/**
	 * Verify the user-provided metadata value.
	 *
	 * @param \LiamW\PostMacros\Entity\Macro $macro The macro being edited or created.
	 * @param mixed $newValue The provided metadata value.
	 *
	 * @return bool True if value is valid, false otherwise.
	 */
	abstract protected function verifyMetadataValue(Macro $macro, &$newValue);

	abstract protected function processInternal(Entity $entity, $value);

	abstract public function getFilterType();

	/**
	 * Get a Macro Metadata handler. Supports short class names and class extensions.
	 *
	 * @param string $shortName The short-form class name.
	 * @param \LiamW\PostMacros\MetaData\Group\AbstractGroup $groupHandler
	 * @param mixed $value The current value of the metadata.
	 *
	 * @return mixed
	 * @throws \Exception
	 */
	public static function create($shortName, $groupHandler, $value = null)
	{
		$className = \XF::stringToClass($shortName, '%s\MetaData\Handler\%s');
		$className = \XF::extendClass($className);

		if (!class_exists($className))
		{
			throw new \InvalidArgumentException("Invalid MetaData class ($shortName) specified.");
		}

		$obj = new $className($value, $groupHandler);

		if ($obj instanceof AbstractHandler)
		{
			return $obj;
		}

		throw new \InvalidArgumentException("Invalid MetaData class ($shortName) specified.");
	}
}