<?php

namespace LiamW\PostMacros\XF\Pub\Controller;

use LiamW\PostMacros\Entity\Macro;
use LiamW\PostMacros\PostMacros;
use XF\Mvc\Entity\ArrayCollection;
use XF\Mvc\ParameterBag;

class Account extends XFCP_Account
{
	//region Extensions
	protected function preferencesSaveProcess(\XF\Entity\User $visitor)
	{
		$form = parent::preferencesSaveProcess($visitor);

		$input = $this->filter([
			'option' => [
				'liamw_postmacros_expanded_macros' => 'bool'
			]
		]);

		$userOptions = $visitor->getRelationOrDefault('Option');
		$form->setupEntityInput($userOptions, $input['option']);

		return $form;
	}
	//endregion

	//region Post Macros
	protected static $macrosPerPage = 25;

	public function actionMacros()
	{
		return $this->redirectPermanently($this->buildLink('account/macros/active'));
	}

	public function actionMacrosActive(ParameterBag $params)
	{
		$page = $this->filterPage($params->page);

		$activeMacrosFinder = $this->getMacroRepo()->findActiveMacrosForList();

		$total = $activeMacrosFinder->total();
		$this->assertValidPage($page, static::$macrosPerPage, $total, 'account/macros/active');

		$macros = $activeMacrosFinder->limitByPage($page, static::$macrosPerPage)->fetch();

		$this->repository('LiamW\PostMacros:Macro')->addSharingsToMacros($macros);

		$viewParams = [
			'page' => $page,
			'perPage' => static::$macrosPerPage,
			'total' => $total,
			'macros' => $macros,
		];

		return $this->addAccountWrapperParams($this->view('LiamW\PostMacros:Macros\List\Active', 'liamw_postmacros_macros_list_active', $viewParams), 'liamw_postmacros_active');
	}

	public function actionMacrosInactive(ParameterBag $params)
	{
		$page = $this->filterPage($params->page);
		$macrosFinder = $this->getMacroRepo()->findInactiveMacroForList();

		$total = $macrosFinder->total();
		$this->assertValidPage($page, static::$macrosPerPage, $total, 'account/macros/inactive');

		$macros = $macrosFinder->limitByPage($page, static::$macrosPerPage)->fetch()->filter(function(Macro $macro)
		{
			return $macro->canEnroll();
		});

		$this->repository('LiamW\PostMacros:Macro')->addSharingsToMacros($macros);

		$viewParams = [
			'page' => $page,
			'perPage' => static::$macrosPerPage,
			'total' => $total,
			'macros' => $macros
		];

		return $this->addAccountWrapperParams($this->view('LiamW\PostMacros:Macros\List\Inactive', 'liamw_postmacros_macros_list_inactive', $viewParams), 'liamw_postmacros_inactive');
	}

	public function actionMacrosNew()
	{
		$this->assertCanCreateMacro();

		$macro = $this->getMacroRepo()->setupBaseMacro();

		return $this->macrosAddEdit($macro);
	}

	public function actionMacrosEdit(ParameterBag $params)
	{
		$macro = $this->assertMacroExists($params->macro_id);
		$this->assertCanEditMacro($macro);

		return $this->macrosAddEdit($macro);
	}

	protected function macrosAddEdit(Macro $macro)
	{
		$viewParams = [
			'macro' => $macro,
			'placeholders' => $this->getMacroRepo()->getMacroPlaceholders(),
			'nextCounter' => $macro->Sharing ? count($macro->Sharing) + 1 : 0
		];

		return $this->addAccountWrapperParams($this->view('LiamW\PostMacros:Macros\Edit', 'liamw_postmacros_macro_edit', $viewParams), 'liamw_postmacros_active');
	}

	protected function macroSaveProcess(Macro $macro)
	{
		$form = $this->formAction();

		$input = $this->filter([
			'title' => 'str',
			'auto_submit' => 'bool',
			'display_order' => 'uint',
			'metadata' => $this->getMacroRepo()->getMetaDataFilters()
		]);
        $type = $this->filter('change_category', 'array');
        $input['change_category'] = $this->getMacroRepo()->getChangeTypeBySave($type, $macro);
		$input['content'] = $this->plugin('XF:Editor')->fromInput('content');

		$form->basicEntitySave($macro, $input);

		if ($macro->canEditSharings())
		{
			$sharings = $this->filter('sharing', 'array');

			$keptSharings = [];

			foreach ($sharings as $sharing)
			{
				if (empty($sharing['username']))
				{
					continue;
				}

				$user = $this->finder('XF:User')->where('username', $sharing['username'])->fetchOne();

				if (!$user)
				{
					throw $this->exception($this->error(\XF::phrase('requested_user_x_not_found', ['name' => $sharing['username']])));
				}

				$userId = $user->user_id;

				if ($userId == $macro->user_id)
				{
					continue;
				}

				$sharingEntity = $macro->getRelationFinder('Sharing')->where('user_id', $userId)->fetchOne();

				if ($sharingEntity)
				{
					$sharingEntity->can_write = isset($sharing['can_write']) ? $sharing['can_write'] : 0;
					$keptSharings[$sharingEntity->user_id] = $sharingEntity;
				}
				else
				{
					$sharingEntity = $macro->getNewSharing();
					$sharingEntity->user_id = $userId;
					$sharingEntity->can_write = isset($sharing['can_write']) ? $sharing['can_write'] : 0;
				}

				$macro->addCascadedSave($sharingEntity);
			}

			/** @var \LiamW\PostMacros\Entity\MacroSharing[]|ArrayCollection $existingSharings */
			$existingSharings = $macro->Sharing;
			$deleteSharedUserIds = array_diff($existingSharings->keys(), array_keys($keptSharings));

			foreach ($deleteSharedUserIds as $userId)
			{
				$existingSharings[$userId]->delete();
			}
		}

		return $form;
	}

	public function actionMacrosSave(ParameterBag $params)
	{
		$this->assertPostOnly();

		if ($params->macro_id)
		{
			$macro = $this->assertMacroExists($params->macro_id);
			$this->assertCanEditMacro($macro);
		}
		else
		{
			$macro = null;
		}

		$macro = $this->getMacroRepo()->setupBaseMacro($macro);
		$this->macroSaveProcess($macro)->run();

		return $this->redirect($this->buildLink('account/macros/'));
	}

	public function actionMacrosDelete(ParameterBag $params)
	{
		$macro = $this->assertMacroExists($params->macro_id);
		if (!$macro->canDelete($error))
		{
			throw $this->exception($this->noPermission($error));
		}

		if ($this->isPost())
		{
			$macro->delete();

			return $this->redirect($this->buildLink('account/macros/active'));
		}
		else
		{
			$viewParams = [
				'macro' => $macro
			];

			return $this->addAccountWrapperParams($this->view('LiamW\PostMacros:Macros\Delete', 'liamw_postmacros_macro_delete', $viewParams), 'liamw_postmacros_active');
		}
	}

	public function actionMacrosUnenroll(ParameterBag $params)
	{
		$user = \XF::visitor();
		$macro = $this->assertMacroExists($params['macro_id'], 'Enrolment|' . $user->user_id);

		if (!$macro->canUnenroll())
		{
			return $this->noPermission();
		}

		$macro->unenroll();

		return $this->redirect($this->buildLink('account/macros/active'));
	}

	public function actionMacrosEnroll(ParameterBag $params)
	{
		$user = \XF::visitor();
		$macro = $this->assertMacroExists($params['macro_id'], 'Enrolment|' . $user->user_id);

		if (!$macro->canEnroll($error))
		{
			return $this->noPermission($error);
		}

		$macro->enroll();

		return $this->redirect($this->buildLink('account/macros/inactive'));
	}

	public function actionMacrosTokens()
	{
		$macroId = $this->filter('macro_id', 'uint');
		$macro = $this->assertMacroExists($macroId);

		if (!$macro->custom_tokens && !$macro->getCategory($this->filter('action_url', 'str')))
		{
			return $this->noPermission();
		}

		$viewParams = [
			'macro' => $macro,
			'editorId' => $this->filter('editor_id', 'str'),
			'actionUrl' => $this->filter('action_url', 'str')
		];

		return $this->view('LiamW\PostMacros:Macros\TokenEntry', 'liamw_postmacros_submit_placeholders', $viewParams);
	}

	public function actionMacrosGet(ParameterBag $params)
	{
		$macroId = $this->filter('macro_id', 'uint');

		if (!$macroId)
		{
			$macroId = $params->macro_id;
		}

		$macro = $this->assertMacroExists($macroId);

		$tokens = $this->filterFormJson([
			'tokens' => 'array-str'
		])['tokens'];
        if($macro->custom_tokens)
        {
            foreach ($macro->custom_tokens AS $key => $token)
            {
                if ($token['required'] && empty($tokens[$key]))
                {
                    return $this->error(\XF::phrase('liamw_postmacros_please_enter_value_for_required_token_x', ['tokenlabel' => $token['label']]));
                }
            }
        }
		$actionUrl = $this->filter('action_url', 'str');
		$macroEntity = $macro->setContextUrl($actionUrl);
        $changeCategory = $this->filter('change_category', 'int');
        $macroEntity = $macroEntity->setCustomTokens($tokens);
        if($changeCategory)
        {
            $macroEntity = $macroEntity->setChangeCategoryId($changeCategory);
        }
		$viewParams = [
			'macro' => $macroEntity
		];

		return $this->view('LiamW\PostMacros:Macros\Get', '', $viewParams);
	}

	/**
	 * @param int $id
	 * @param null $with
	 * @param null $phraseKey
	 *
	 * @return \LiamW\PostMacros\Entity\Macro
	 */
	protected function assertMacroExists($id, $with = null, $phraseKey = null)
	{
		/** @var Macro $macro */
		$macro = $this->assertRecordExists('LiamW\PostMacros:Macro', $id, $with, $phraseKey);

		return $macro;
	}

	/**
	 * @param \LiamW\PostMacros\Entity\Macro $macro
	 *
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertCanEditMacro($macro)
	{
		if (!$macro->canEdit($error))
		{
			throw $this->exception($this->noPermission($error));
		}
	}

	protected function assertCanCreateMacro()
	{
		if (!PostMacros::visitor()->canCreateMacro($error))
		{
			throw $this->exception($this->noPermission($error));
		}
	}

	/**
	 * @return \LiamW\PostMacros\Repository\Macro|\XF\Mvc\Entity\Repository
	 */
	protected function getMacroRepo()
	{
		return $this->repository('LiamW\PostMacros:Macro');
	}
	//endregion
}